@@ -2,38 +2,76 @@
 
 import gettext
 gettext.install("pearlupdate", "/usr/share/locale")
-
 import html
-
 from gi.repository import Gio
 
+
+# These updates take priority over other updates.
+# If a new version of these packages is available, nothing else is listed.
+PRIORITY_UPDATES = ['pearlupdate', 'pearl-upgrade-info']
+
 settings = Gio.Settings("com.pearllinux.updates")
-if settings.get_boolean("use-lowlatency-kernels"):
-    KERNEL_PKG_NAMES = ['linux-headers-VERSION', 'linux-headers-VERSION-lowlatency', 'linux-image-VERSION-lowlatency', 'linux-modules-VERSION-lowlatency', 'linux-modules-extra-VERSION-lowlatency']
-    KERNEL_PKG_NAMES.append('linux-image-extra-VERSION-lowlatency') # Naming convention in 16.04, until 4.15 series
-    META_NAMES = ['linux-lowlatency', 'linux-lowlatency-hwe-16.04']
-else:
-    KERNEL_PKG_NAMES = ['linux-headers-VERSION', 'linux-headers-VERSION-generic', 'linux-image-VERSION-generic', 'linux-modules-VERSION-generic', 'linux-modules-extra-VERSION-generic']
-    KERNEL_PKG_NAMES.append('linux-image-extra-VERSION-generic') # Naming convention in 16.04, until 4.15 series
-    META_NAMES = ['linux-generic', 'linux-generic-hwe-16.04']
 
-class Rule():
+SUPPORTED_KERNEL_TYPES = ["-generic", "-lowlatency", "-aws", "-azure", "-gcp", "-kvm", "-oem", "-oracle"]
+KERNEL_PKG_NAMES = ['linux-headers-VERSION', 'linux-headers-VERSION-KERNELTYPE', 'linux-image-VERSION-KERNELTYPE', \
+    'linux-modules-VERSION-KERNELTYPE', 'linux-modules-extra-VERSION-KERNELTYPE']
+KERNEL_PKG_NAMES.append('linux-image-extra-VERSION-KERNELTYPE') # Naming convention in 16.04, until 4.15 series
 
-    def __init__(self, name, version, level):
-        self.name = name
+CONFIGURED_KERNEL_TYPE = settings.get_string("selected-kernel-type")
+if CONFIGURED_KERNEL_TYPE not in SUPPORTED_KERNEL_TYPES:
+    CONFIGURED_KERNEL_TYPE = "-generic"
+
+def get_release_dates():
+    """ Get distro release dates for support duration calculation """
+    import os
+    import time
+    from datetime import datetime
+
+    release_dates = {}
+    distro_info = []
+    if os.path.isfile("/usr/share/distro-info/ubuntu.csv"):
+        distro_info += open("/usr/share/distro-info/ubuntu.csv", "r").readlines()
+    if os.path.isfile("/usr/share/distro-info/debian.csv"):
+        distro_info += open("/usr/share/distro-info/debian.csv", "r").readlines()
+    if distro_info:
+        for distro in distro_info[1:]:
+            try:
+                distro = distro.split(",")
+                release_date = time.mktime(time.strptime(distro[4], '%Y-%m-%d'))
+                release_date = datetime.fromtimestamp(release_date)
+                support_end = time.mktime(time.strptime(distro[5].rstrip(), '%Y-%m-%d'))
+                support_end = datetime.fromtimestamp(support_end)
+                release_dates[distro[2]] = [release_date, support_end]
+            except:
+                pass
+    return release_dates
+
+class KernelVersion():
+
+    def __init__(self, version):
+        field_length = 3
         self.version = version
-        self.level = level
-        self.is_wildcard = self.name.startswith("*")
-        if self.is_wildcard:
-            self.name = self.name.replace("*", "")
-
-    def match(self, pkg_name, pkg_version):
-        if (self.version == "*" or self.version == pkg_version):
-            if self.is_wildcard:
-                return self.name in pkg_name
-            else:
-                return pkg_name == self.name
-        return False
+        self.version_id = []
+        version_id = self.version.replace("-", ".").split(".")
+        # Check if mainline rc kernel to ensure proper sorting vs mainline release kernels
+        suffix = next((x for x in version_id if x.startswith("rc")), None)
+        if not suffix:
+            suffix = "z"
+        # Copy numeric parts from version_id to self.version_id and fill up to field_length
+        for element in version_id:
+            if element.isnumeric():
+                self.version_id.append("0" * (field_length - len(element)) + element)
+        # Installed kernels always have len(self.version_id) >= 4 at this point,
+        # create missing parts for not installed mainline kernels:
+        while len(self.version_id) < 3:
+            self.version_id.append("0" * field_length)
+        if len(self.version_id) == 3:
+            self.version_id.append(f"{''.join((x[:field_length - 2].lstrip('0') + x[field_length - 2:] for x in self.version_id))}{suffix}")
+        elif len(self.version_id[3]) == 6:
+            # installed release mainline kernel, add suffix for sorting
+            self.version_id[3] += suffix
+        self.series = tuple(self.version_id[:3])
+        self.shortseries = tuple(self.version_id[:2])
 
 class Update():
 
@@ -41,6 +79,7 @@ class Update():
         self.package_names = []
         if package is not None:
             self.package_names.append(package.name)
+            self.source_packages = {f"{package.candidate.source_name}={package.candidate.source_version}"}
             self.main_package_name = package.name
             self.package_name = package.name
             self.new_version = package.candidate.version
@@ -57,12 +96,14 @@ class Update():
             self.display_name = self.source_name
             self.short_description = package.candidate.raw_description
             self.description = package.candidate.description
+            self.archive = ""
             if (self.new_version != self.old_version):
                 self.type = "package"
                 self.origin = ""
                 for origin in package.candidate.origins:
                     self.origin = origin.origin
                     self.site = origin.site
+                    self.archive = origin.archive
                     if origin.origin == "Ubuntu":
                         self.origin = "ubuntu"
                     elif origin.origin == "Debian":
@@ -82,16 +123,15 @@ class Update():
                         if origin.component == "romeo":
                             self.type = "unstable"
                             break
-                if self.source_name in ["linux", "linux-kernel"]:
+                if package.candidate.section == "kernel" or self.package_name.startswith("linux-headers") or self.real_source_name in ["linux", "linux-kernel", "linux-signed", "linux-meta"]:
                     self.type = "kernel"
-
-            self.level = 2 # Level 2 by default
         else:
             # Build the class from the input_string
             self.parse(input_string)
 
     def add_package(self, pkg):
         self.package_names.append(pkg.name)
+        self.source_packages.add(f"{pkg.candidate.source_name}={pkg.candidate.source_version}")
         self.size += pkg.candidate.size
         if self.main_package_name is None or pkg.name == self.source_name:
             self.overwrite_main_package(pkg)
@@ -116,10 +156,14 @@ class Update():
     def overwrite_main_package(self, pkg):
         self.description = pkg.candidate.description
         self.short_description = pkg.candidate.raw_description
-        self.main_package_name  = pkg.name
+        self.main_package_name = pkg.name
 
     def serialize(self):
-        output_string = u"###%d###%s###%s###%s###%s###%s###%s###%s###%s###%s###%s###%s###%s###%s---EOL---" % (self.level, self.display_name, self.source_name, self.real_source_name, self.main_package_name, ", ".join(self.package_names), self.new_version, self.old_version, self.size, self.type, self.origin, self.short_description, self.description, self.site)
+        output_string = u"###%s###%s###%s###%s###%s###%s###%s###%s###%s###%s###%s###%s###%s###%s###%s---EOL---" % \
+        (self.display_name, self.source_name, self.real_source_name, ", ".join(self.source_packages),
+         self.main_package_name, ", ".join(self.package_names), self.new_version,
+         self.old_version, self.size, self.type, self.origin,
+         self.short_description, self.description, self.site, self.archive)
         print(output_string.encode('ascii', 'xmlcharrefreplace'))
 
     def parse(self, input_string):
@@ -127,12 +171,14 @@ class Update():
             input_string = html.unescape(input_string)
         except:
             pass
-        values = input_string.split("###")
-        nothing, level, self.display_name, self.source_name, self.real_source_name, self.main_package_name, package_names, self.new_version, self.old_version, size, self.type, self.origin, self.short_description, self.description, self.site = values
-        self.level = int(level)
+        values = input_string.split("###")[1:]
+        (self.display_name, self.source_name, self.real_source_name, source_packages,
+         self.main_package_name, package_names, self.new_version,
+         self.old_version, size, self.type, self.origin, self.short_description,
+         self.description, self.site, self.archive) = values
         self.size = int(size)
-        for package_name in package_names.split(", "):
-            self.package_names.append(package_name)
+        self.package_names = package_names.split(", ")
+        self.source_packages = source_packages.split(", ")
 
 class Alias():
     def __init__(self, name, short_description, description):
@@ -150,4 +196,4 @@ class Alias():
 
         self.name = name
         self.short_description = short_description
-        self.description = description
\ No newline at end of file
+        self.description = description
